@@ -10,6 +10,7 @@
  * B.P. 105 - 78153 Le Chesnay, France
  */
 
+#include <isl/id.h>
 #include <isl/val.h>
 #include <isl/space.h>
 #include <isl/map.h>
@@ -56,13 +57,17 @@ static __isl_give isl_ast_build *isl_ast_build_init_derived(
 {
 	isl_ctx *ctx;
 	isl_vec *strides;
+	isl_size dim;
 
 	build = isl_ast_build_cow(build);
 	if (!build || !build->domain)
 		goto error;
 
 	ctx = isl_ast_build_get_ctx(build);
-	strides = isl_vec_alloc(ctx, isl_space_dim(space, isl_dim_set));
+	dim = isl_space_dim(space, isl_dim_set);
+	if (dim < 0)
+		goto error;
+	strides = isl_vec_alloc(ctx, dim);
 	strides = isl_vec_set_si(strides, 1);
 
 	isl_vec_free(build->strides);
@@ -113,14 +118,16 @@ static __isl_give isl_id *generate_name(isl_ctx *ctx, int i,
  */
 __isl_give isl_ast_build *isl_ast_build_from_context(__isl_take isl_set *set)
 {
-	int i, n;
+	int i;
+	isl_size n;
 	isl_ctx *ctx;
 	isl_space *space;
 	isl_ast_build *build;
 
 	set = isl_set_compute_divs(set);
-	if (!set)
-		return NULL;
+	n = isl_set_dim(set, isl_dim_set);
+	if (n < 0)
+		goto error;
 
 	ctx = isl_set_get_ctx(set);
 
@@ -133,7 +140,6 @@ __isl_give isl_ast_build *isl_ast_build_from_context(__isl_take isl_set *set)
 	build->generated = isl_set_copy(build->domain);
 	build->pending = isl_set_universe(isl_set_get_space(build->domain));
 	build->options = isl_union_map_empty(isl_space_params_alloc(ctx, 0));
-	n = isl_set_dim(set, isl_dim_set);
 	build->depth = n;
 	build->iterators = isl_id_list_alloc(ctx, n);
 	for (i = 0; i < n; ++i) {
@@ -357,14 +363,16 @@ error:
 __isl_give isl_ast_build *isl_ast_build_set_iterators(
 	__isl_take isl_ast_build *build, __isl_take isl_id_list *iterators)
 {
-	int dim, n_it;
+	isl_size dim, n_it;
 
 	build = isl_ast_build_cow(build);
 	if (!build)
 		goto error;
 
-	dim = isl_set_dim(build->domain, isl_dim_set);
+	dim = isl_ast_build_dim(build, isl_dim_set);
 	n_it = isl_id_list_n_id(build->iterators);
+	if (dim < 0 || n_it < 0)
+		goto error;
 	if (n_it < dim)
 		isl_die(isl_ast_build_get_ctx(build), isl_error_internal,
 			"isl_ast_build in inconsistent state", goto error);
@@ -561,15 +569,14 @@ static void isl_ast_build_reset_schedule_map(__isl_keep isl_ast_build *build)
  * if code has been generated for the entire schedule and if none
  * of the loops have been eliminated.
  */
-__isl_give int isl_ast_build_need_schedule_map(__isl_keep isl_ast_build *build)
+isl_bool isl_ast_build_need_schedule_map(__isl_keep isl_ast_build *build)
 {
-	int dim;
+	isl_size dim;
 
-	if (!build)
-		return -1;
-
-	dim = isl_set_dim(build->domain, isl_dim_set);
-	return build->depth != dim || any_eliminated(build);
+	dim = isl_ast_build_dim(build, isl_dim_set);
+	if (dim < 0)
+		return isl_bool_error;
+	return isl_bool_ok(build->depth != dim || any_eliminated(build));
 }
 
 /* Return a mapping from the internal schedule space to the external
@@ -593,6 +600,7 @@ __isl_give int isl_ast_build_need_schedule_map(__isl_keep isl_ast_build *build)
 __isl_give isl_multi_aff *isl_ast_build_get_schedule_map_multi_aff(
 	__isl_keep isl_ast_build *build)
 {
+	isl_bool needs_map;
 	isl_space *space;
 	isl_multi_aff *ma;
 
@@ -600,13 +608,19 @@ __isl_give isl_multi_aff *isl_ast_build_get_schedule_map_multi_aff(
 		return NULL;
 	if (build->schedule_map)
 		return isl_multi_aff_copy(build->schedule_map);
+	needs_map = isl_ast_build_need_schedule_map(build);
+	if (needs_map < 0)
+		return NULL;
 
 	space = isl_ast_build_get_space(build, 1);
 	space = isl_space_map_from_set(space);
 	ma = isl_multi_aff_identity(space);
-	if (isl_ast_build_need_schedule_map(build)) {
+	if (needs_map) {
 		int i;
-		int dim = isl_set_dim(build->domain, isl_dim_set);
+		isl_size dim = isl_ast_build_dim(build, isl_dim_set);
+
+		if (dim < 0)
+			ma = isl_multi_aff_free(ma);
 		ma = isl_multi_aff_drop_dims(ma, isl_dim_out,
 					build->depth, dim - build->depth);
 		for (i = build->depth - 1; i >= 0; --i)
@@ -761,7 +775,8 @@ static __isl_give isl_set *intersect_stride_constraint(__isl_take isl_set *set,
 static __isl_give isl_ast_build *update_values(
 	__isl_take isl_ast_build *build, __isl_take isl_basic_set *bounds)
 {
-	int sv;
+	isl_bool sv;
+	isl_size n;
 	isl_pw_multi_aff *pma;
 	isl_aff *aff = NULL;
 	isl_map *it_map;
@@ -786,10 +801,10 @@ static __isl_give isl_ast_build *update_values(
 	build->value = isl_pw_aff_coalesce(build->value);
 	isl_pw_multi_aff_free(pma);
 
-	if (!build->value)
+	n = isl_pw_aff_n_piece(build->value);
+	if (n < 0)
 		return isl_ast_build_free(build);
-
-	if (isl_pw_aff_n_piece(build->value) != 1)
+	if (n != 1)
 		return build;
 
 	isl_pw_aff_foreach_piece(build->value, &extract_single_piece, &aff);
@@ -991,15 +1006,24 @@ __isl_give isl_ast_build *isl_ast_build_replace_pending_by_guard(
 __isl_give isl_ast_build *isl_ast_build_restrict(
 	__isl_take isl_ast_build *build, __isl_take isl_set *set)
 {
+	isl_bool needs_map;
+
 	if (isl_set_is_params(set))
 		return isl_ast_build_restrict_generated(build, set);
 
-	if (isl_ast_build_need_schedule_map(build)) {
+	needs_map = isl_ast_build_need_schedule_map(build);
+	if (needs_map < 0)
+		goto error;
+	if (needs_map) {
 		isl_multi_aff *ma;
 		ma = isl_ast_build_get_schedule_map_multi_aff(build);
 		set = isl_set_preimage_multi_aff(set, ma);
 	}
 	return isl_ast_build_restrict_generated(build, set);
+error:
+	isl_ast_build_free(build);
+	isl_set_free(set);
+	return NULL;
 }
 
 /* Replace build->executed by "executed".
@@ -1048,18 +1072,22 @@ static __isl_give isl_ast_build *extract_loop_types(
 	__isl_take isl_ast_build *build)
 {
 	int i;
+	isl_size n;
 	isl_ctx *ctx;
 	isl_schedule_node *node;
 
 	if (!build)
 		return NULL;
+	n = isl_schedule_node_band_n_member(build->node);
+	if (n < 0)
+		return isl_ast_build_free(build);
 	ctx = isl_ast_build_get_ctx(build);
 	if (!build->node)
 		isl_die(ctx, isl_error_internal, "missing AST node",
 			return isl_ast_build_free(build));
 
 	free(build->loop_type);
-	build->n = isl_schedule_node_band_n_member(build->node);
+	build->n = n;
 	build->loop_type = isl_alloc_array(ctx,
 					    enum isl_ast_loop_type, build->n);
 	if (build->n && !build->loop_type)
@@ -1145,11 +1173,11 @@ __isl_give isl_multi_aff *isl_ast_build_get_internal2input(
 /* Return the number of variables of the given type
  * in the (internal) schedule space.
  */
-unsigned isl_ast_build_dim(__isl_keep isl_ast_build *build,
+isl_size isl_ast_build_dim(__isl_keep isl_ast_build *build,
 	enum isl_dim_type type)
 {
 	if (!build)
-		return 0;
+		return isl_size_error;
 	return isl_set_dim(build->domain, type);
 }
 
@@ -1166,7 +1194,8 @@ __isl_give isl_space *isl_ast_build_get_space(__isl_keep isl_ast_build *build,
 	int internal)
 {
 	int i;
-	int dim;
+	isl_size dim;
+	isl_bool needs_map;
 	isl_space *space;
 
 	if (!build)
@@ -1176,10 +1205,15 @@ __isl_give isl_space *isl_ast_build_get_space(__isl_keep isl_ast_build *build,
 	if (internal)
 		return space;
 
-	if (!isl_ast_build_need_schedule_map(build))
+	needs_map = isl_ast_build_need_schedule_map(build);
+	if (needs_map < 0)
+		return isl_space_free(space);
+	if (!needs_map)
 		return space;
 
-	dim = isl_set_dim(build->domain, isl_dim_set);
+	dim = isl_ast_build_dim(build, isl_dim_set);
+	if (dim < 0)
+		return isl_space_free(space);
 	space = isl_space_drop_dims(space, isl_dim_set,
 				    build->depth, dim - build->depth);
 	for (i = build->depth - 1; i >= 0; --i) {
@@ -1231,14 +1265,16 @@ __isl_give isl_space *isl_ast_build_get_schedule_space(
 __isl_give isl_union_map *isl_ast_build_get_schedule(
 	__isl_keep isl_ast_build *build)
 {
+	isl_bool needs_map;
 	isl_union_map *executed;
 	isl_union_map *schedule;
 
-	if (!build)
+	needs_map = isl_ast_build_need_schedule_map(build);
+	if (needs_map < 0)
 		return NULL;
 
 	executed = isl_union_map_copy(build->executed);
-	if (isl_ast_build_need_schedule_map(build)) {
+	if (needs_map) {
 		isl_map *proj = isl_ast_build_get_schedule_map(build);
 		executed = isl_union_map_apply_domain(executed,
 					isl_union_map_from_map(proj));
@@ -1819,7 +1855,7 @@ __isl_give isl_ast_build *isl_ast_build_product(
 	isl_vec *strides;
 	isl_set *set;
 	isl_multi_aff *embedding;
-	int dim, n_it;
+	isl_size dim, space_dim, n_it;
 
 	build = isl_ast_build_cow(build);
 	if (!build)
@@ -1828,9 +1864,12 @@ __isl_give isl_ast_build *isl_ast_build_product(
 	build->outer_pos = build->depth;
 
 	ctx = isl_ast_build_get_ctx(build);
-	dim = isl_set_dim(build->domain, isl_dim_set);
-	dim += isl_space_dim(space, isl_dim_set);
+	dim = isl_ast_build_dim(build, isl_dim_set);
+	space_dim = isl_space_dim(space, isl_dim_set);
 	n_it = isl_id_list_n_id(build->iterators);
+	if (dim < 0 || space_dim < 0 || n_it < 0)
+		goto error;
+	dim += space_dim;
 	if (n_it < dim) {
 		isl_id_list *l;
 		l = generate_names(ctx, dim - n_it, n_it, build);
@@ -1845,7 +1884,7 @@ __isl_give isl_ast_build *isl_ast_build_product(
 	build->pending = isl_set_product(build->pending, isl_set_copy(set));
 	build->generated = isl_set_product(build->generated, set);
 
-	strides = isl_vec_alloc(ctx, isl_space_dim(space, isl_dim_set));
+	strides = isl_vec_alloc(ctx, space_dim);
 	strides = isl_vec_set_si(strides, 1);
 	build->strides = isl_vec_concat(build->strides, strides);
 
