@@ -21,7 +21,6 @@
 #include <isl_vec_private.h>
 #include <isl_lp_private.h>
 #include <isl_ilp_private.h>
-#include <isl/deprecated/ilp_int.h>
 
 /* Given a basic set "bset", construct a basic set U such that for
  * each element x in U, the whole unit box positioned at x is inside
@@ -32,11 +31,12 @@
  * term.  This ensures that if x satisfies the resulting constraints,
  * then x plus any sum of unit vectors satisfies the original constraints.
  */
-static struct isl_basic_set *unit_box_base_points(struct isl_basic_set *bset)
+static __isl_give isl_basic_set *unit_box_base_points(
+	__isl_take isl_basic_set *bset)
 {
 	int i, j, k;
 	struct isl_basic_set *unit_box = NULL;
-	unsigned total;
+	isl_size total;
 
 	if (!bset)
 		goto error;
@@ -47,7 +47,9 @@ static struct isl_basic_set *unit_box_base_points(struct isl_basic_set *bset)
 		return isl_basic_set_empty(space);
 	}
 
-	total = isl_basic_set_total_dim(bset);
+	total = isl_basic_set_dim(bset, isl_dim_all);
+	if (total < 0)
+		goto error;
 	unit_box = isl_basic_set_alloc_space(isl_basic_set_get_space(bset),
 					0, 0, bset->n_ineq);
 
@@ -80,7 +82,8 @@ error:
  * and round it up to the nearest integer.
  * If not, we simply pick any integer point in "bset".
  */
-static struct isl_vec *initial_solution(struct isl_basic_set *bset, isl_int *f)
+static __isl_give isl_vec *initial_solution(__isl_keep isl_basic_set *bset,
+	isl_int *f)
 {
 	enum isl_lp_result res;
 	struct isl_basic_set *unit_box;
@@ -102,13 +105,15 @@ static struct isl_vec *initial_solution(struct isl_basic_set *bset, isl_int *f)
 
 /* Restrict "bset" to those points with values for f in the interval [l, u].
  */
-static struct isl_basic_set *add_bounds(struct isl_basic_set *bset,
+static __isl_give isl_basic_set *add_bounds(__isl_take isl_basic_set *bset,
 	isl_int *f, isl_int l, isl_int u)
 {
 	int k;
-	unsigned total;
+	isl_size total;
 
-	total = isl_basic_set_total_dim(bset);
+	total = isl_basic_set_dim(bset, isl_dim_all);
+	if (total < 0)
+		return isl_basic_set_free(bset);
 	bset = isl_basic_set_extend_constraints(bset, 0, 2);
 
 	k = isl_basic_set_alloc_inequality(bset);
@@ -145,8 +150,8 @@ error:
  * If no point can be found, we update l to the upper bound of the interval
  * we checked (u or l+floor(u-l-1/2)) plus 1.
  */
-static struct isl_vec *solve_ilp_search(struct isl_basic_set *bset,
-	isl_int *f, isl_int *opt, struct isl_vec *sol, isl_int l, isl_int u)
+static __isl_give isl_vec *solve_ilp_search(__isl_keep isl_basic_set *bset,
+	isl_int *f, isl_int *opt, __isl_take isl_vec *sol, isl_int l, isl_int u)
 {
 	isl_int tmp;
 	int divide = 1;
@@ -204,9 +209,8 @@ static struct isl_vec *solve_ilp_search(struct isl_basic_set *bset,
  *
  * We then call solve_ilp_search to perform a binary search on the interval.
  */
-static enum isl_lp_result solve_ilp(struct isl_basic_set *bset,
-				      isl_int *f, isl_int *opt,
-				      struct isl_vec **sol_p)
+static enum isl_lp_result solve_ilp(__isl_keep isl_basic_set *bset,
+	isl_int *f, isl_int *opt, __isl_give isl_vec **sol_p)
 {
 	enum isl_lp_result res;
 	isl_int l, u;
@@ -260,17 +264,18 @@ static enum isl_lp_result solve_ilp(struct isl_basic_set *bset,
 	return res;
 }
 
-static enum isl_lp_result solve_ilp_with_eq(struct isl_basic_set *bset, int max,
-				      isl_int *f, isl_int *opt,
-				      struct isl_vec **sol_p)
+static enum isl_lp_result solve_ilp_with_eq(__isl_keep isl_basic_set *bset,
+	int max, isl_int *f, isl_int *opt, __isl_give isl_vec **sol_p)
 {
-	unsigned dim;
+	isl_size dim;
 	enum isl_lp_result res;
 	struct isl_mat *T = NULL;
 	struct isl_vec *v;
 
 	bset = isl_basic_set_copy(bset);
-	dim = isl_basic_set_total_dim(bset);
+	dim = isl_basic_set_dim(bset, isl_dim_all);
+	if (dim < 0)
+		goto error;
 	v = isl_vec_alloc(bset->ctx, 1 + dim);
 	if (!v)
 		goto error;
@@ -303,19 +308,17 @@ error:
  * If there is any equality among the points in "bset", then we first
  * project it out.  Otherwise, we continue with solve_ilp above.
  */
-enum isl_lp_result isl_basic_set_solve_ilp(struct isl_basic_set *bset, int max,
-				      isl_int *f, isl_int *opt,
-				      struct isl_vec **sol_p)
+enum isl_lp_result isl_basic_set_solve_ilp(__isl_keep isl_basic_set *bset,
+	int max, isl_int *f, isl_int *opt, __isl_give isl_vec **sol_p)
 {
-	unsigned dim;
+	isl_size dim;
 	enum isl_lp_result res;
 
-	if (!bset)
-		return isl_lp_error;
 	if (sol_p)
 		*sol_p = NULL;
 
-	isl_assert(bset->ctx, isl_basic_set_n_param(bset) == 0, goto error);
+	if (isl_basic_set_check_no_params(bset) < 0)
+		return isl_lp_error;
 
 	if (isl_basic_set_plain_is_empty(bset))
 		return isl_lp_empty;
@@ -323,7 +326,9 @@ enum isl_lp_result isl_basic_set_solve_ilp(struct isl_basic_set *bset, int max,
 	if (bset->n_eq)
 		return solve_ilp_with_eq(bset, max, f, opt, sol_p);
 
-	dim = isl_basic_set_total_dim(bset);
+	dim = isl_basic_set_dim(bset, isl_dim_all);
+	if (dim < 0)
+		return isl_lp_error;
 
 	if (max)
 		isl_seq_neg(f, f, 1 + dim);
@@ -336,9 +341,6 @@ enum isl_lp_result isl_basic_set_solve_ilp(struct isl_basic_set *bset, int max,
 	}
 
 	return res;
-error:
-	isl_basic_set_free(bset);
-	return isl_lp_error;
 }
 
 static enum isl_lp_result basic_set_opt(__isl_keep isl_basic_set *bset, int max,
@@ -355,23 +357,6 @@ static enum isl_lp_result basic_set_opt(__isl_keep isl_basic_set *bset, int max,
 	return res;
 }
 
-static __isl_give isl_mat *extract_divs(__isl_keep isl_basic_set *bset)
-{
-	int i;
-	isl_ctx *ctx = isl_basic_set_get_ctx(bset);
-	isl_mat *div;
-
-	div = isl_mat_alloc(ctx, bset->n_div,
-			    1 + 1 + isl_basic_set_total_dim(bset));
-	if (!div)
-		return NULL;
-
-	for (i = 0; i < bset->n_div; ++i)
-		isl_seq_cpy(div->row[i], bset->div[i], div->n_col);
-
-	return div;
-}
-
 enum isl_lp_result isl_basic_set_opt(__isl_keep isl_basic_set *bset, int max,
 	__isl_keep isl_aff *obj, isl_int *opt)
 {
@@ -381,7 +366,7 @@ enum isl_lp_result isl_basic_set_opt(__isl_keep isl_basic_set *bset, int max,
 	isl_mat *bset_div = NULL;
 	isl_mat *div = NULL;
 	enum isl_lp_result res;
-	int bset_n_div, obj_n_div;
+	isl_size bset_n_div, obj_n_div;
 
 	if (!bset || !obj)
 		return isl_lp_error;
@@ -397,13 +382,15 @@ enum isl_lp_result isl_basic_set_opt(__isl_keep isl_basic_set *bset, int max,
 
 	bset_n_div = isl_basic_set_dim(bset, isl_dim_div);
 	obj_n_div = isl_aff_dim(obj, isl_dim_div);
+	if (bset_n_div < 0 || obj_n_div < 0)
+		return isl_lp_error;
 	if (bset_n_div == 0 && obj_n_div == 0)
 		return basic_set_opt(bset, max, obj, opt);
 
 	bset = isl_basic_set_copy(bset);
 	obj = isl_aff_copy(obj);
 
-	bset_div = extract_divs(bset);
+	bset_div = isl_basic_set_get_divs(bset);
 	exp1 = isl_alloc_array(ctx, int, bset_n_div);
 	exp2 = isl_alloc_array(ctx, int, obj_n_div);
 	if (!bset_div || (bset_n_div && !exp1) || (obj_n_div && !exp2))
@@ -485,12 +472,15 @@ enum isl_lp_result isl_set_opt(__isl_keep isl_set *set, int max,
 	__isl_keep isl_aff *obj, isl_int *opt)
 {
 	enum isl_lp_result res;
+	isl_bool aligned;
 
 	if (!set || !obj)
 		return isl_lp_error;
 
-	if (isl_space_match(set->dim, isl_dim_param,
-			    obj->ls->dim, isl_dim_param))
+	aligned = isl_set_space_has_equal_params(set, obj->ls->dim);
+	if (aligned < 0)
+		return isl_lp_error;
+	if (aligned)
 		return isl_set_opt_aligned(set, max, obj, opt);
 
 	set = isl_set_copy(set);
@@ -506,24 +496,6 @@ enum isl_lp_result isl_set_opt(__isl_keep isl_set *set, int max,
 	return res;
 }
 
-enum isl_lp_result isl_basic_set_max(__isl_keep isl_basic_set *bset,
-	__isl_keep isl_aff *obj, isl_int *opt)
-{
-	return isl_basic_set_opt(bset, 1, obj, opt);
-}
-
-enum isl_lp_result isl_set_max(__isl_keep isl_set *set,
-	__isl_keep isl_aff *obj, isl_int *opt)
-{
-	return isl_set_opt(set, 1, obj, opt);
-}
-
-enum isl_lp_result isl_set_min(__isl_keep isl_set *set,
-	__isl_keep isl_aff *obj, isl_int *opt)
-{
-	return isl_set_opt(set, 0, obj, opt);
-}
-
 /* Convert the result of a function that returns an isl_lp_result
  * to an isl_val.  The numerator of "v" is set to the optimal value
  * if lp_res is isl_lp_ok.  "max" is set if a maximum was computed.
@@ -670,15 +642,13 @@ error:
 	return NULL;
 }
 
-/* Internal data structure for isl_set_opt_pw_aff.
+/* Internal data structure for isl_pw_aff_opt_val.
  *
  * "max" is set if the maximum should be computed.
- * "set" is the set over which the optimum should be computed.
  * "res" contains the current optimum and is initialized to NaN.
  */
-struct isl_set_opt_data {
+struct isl_pw_aff_opt_data {
 	int max;
-	isl_set *set;
 
 	isl_val *res;
 };
@@ -689,10 +659,9 @@ struct isl_set_opt_data {
 static isl_stat piece_opt(__isl_take isl_set *set, __isl_take isl_aff *aff,
 	void *user)
 {
-	struct isl_set_opt_data *data = user;
+	struct isl_pw_aff_opt_data *data = user;
 	isl_val *opt;
 
-	set = isl_set_intersect(set, isl_set_copy(data->set));
 	opt = isl_set_opt_val(set, data->max, aff);
 	isl_set_free(set);
 	isl_aff_free(aff);
@@ -705,57 +674,46 @@ static isl_stat piece_opt(__isl_take isl_set *set, __isl_take isl_aff *aff,
 }
 
 /* Return the minimum (maximum if "max" is set) of the integer piecewise affine
- * expression "obj" over the points in "set".
+ * expression "pa" over its definition domain.
  *
  * Return infinity or negative infinity if the optimal value is unbounded and
- * NaN if the intersection of "set" with the domain of "obj" is empty.
+ * NaN if the domain of "pa" is empty.
  *
  * Initialize the result to NaN and then update it for each of the pieces
- * in "obj".
+ * in "pa".
  */
-static __isl_give isl_val *isl_set_opt_pw_aff(__isl_keep isl_set *set, int max,
-	__isl_keep isl_pw_aff *obj)
+static __isl_give isl_val *isl_pw_aff_opt_val(__isl_take isl_pw_aff *pa,
+	int max)
 {
-	struct isl_set_opt_data data = { max, set };
+	struct isl_pw_aff_opt_data data = { max };
 
-	data.res = isl_val_nan(isl_set_get_ctx(set));
-	if (isl_pw_aff_foreach_piece(obj, &piece_opt, &data) < 0)
-		return isl_val_free(data.res);
+	data.res = isl_val_nan(isl_pw_aff_get_ctx(pa));
+	if (isl_pw_aff_foreach_piece(pa, &piece_opt, &data) < 0)
+		data.res = isl_val_free(data.res);
 
+	isl_pw_aff_free(pa);
 	return data.res;
 }
 
-/* Internal data structure for isl_union_set_opt_union_pw_aff.
+/* Internal data structure for isl_union_pw_aff_opt_val.
  *
  * "max" is set if the maximum should be computed.
- * "obj" is the objective function that needs to be optimized.
  * "res" contains the current optimum and is initialized to NaN.
  */
-struct isl_union_set_opt_data {
+struct isl_union_pw_aff_opt_data {
 	int max;
-	isl_union_pw_aff *obj;
 
 	isl_val *res;
 };
 
-/* Update the optimum in data->res with the optimum over "set".
- * Do so by first extracting the matching objective function
- * from data->obj.
+/* Update the optimum in data->res with the optimum of "pa".
  */
-static isl_stat set_opt(__isl_take isl_set *set, void *user)
+static isl_stat pw_aff_opt(__isl_take isl_pw_aff *pa, void *user)
 {
-	struct isl_union_set_opt_data *data = user;
-	isl_space *space;
-	isl_pw_aff *pa;
+	struct isl_union_pw_aff_opt_data *data = user;
 	isl_val *opt;
 
-	space = isl_set_get_space(set);
-	space = isl_space_from_domain(space);
-	space = isl_space_add_dims(space, isl_dim_out, 1);
-	pa = isl_union_pw_aff_extract_pw_aff(data->obj, space);
-	opt = isl_set_opt_pw_aff(set, data->max, pa);
-	isl_pw_aff_free(pa);
-	isl_set_free(set);
+	opt = isl_pw_aff_opt_val(pa, data->max);
 
 	data->res = val_opt(data->res, opt, data->max);
 	if (!data->res)
@@ -765,27 +723,86 @@ static isl_stat set_opt(__isl_take isl_set *set, void *user)
 }
 
 /* Return the minimum (maximum if "max" is set) of the integer piecewise affine
- * expression "obj" over the points in "uset".
+ * expression "upa" over its definition domain.
  *
  * Return infinity or negative infinity if the optimal value is unbounded and
- * NaN if the intersection of "uset" with the domain of "obj" is empty.
+ * NaN if the domain of the expression is empty.
  *
- * Initialize the result to NaN and then update it for each of the sets
- * in "uset".
+ * Initialize the result to NaN and then update it
+ * for each of the piecewise affine expressions in "upa".
  */
-static __isl_give isl_val *isl_union_set_opt_union_pw_aff(
-	__isl_keep isl_union_set *uset, int max,
-	__isl_keep isl_union_pw_aff *obj)
+static __isl_give isl_val *isl_union_pw_aff_opt_val(
+	__isl_take isl_union_pw_aff *upa, int max)
 {
-	struct isl_union_set_opt_data data = { max, obj };
+	struct isl_union_pw_aff_opt_data data = { max };
 
-	data.res = isl_val_nan(isl_union_set_get_ctx(uset));
-	if (isl_union_set_foreach_set(uset, &set_opt, &data) < 0)
-		return isl_val_free(data.res);
+	data.res = isl_val_nan(isl_union_pw_aff_get_ctx(upa));
+	if (isl_union_pw_aff_foreach_pw_aff(upa, &pw_aff_opt, &data) < 0)
+		data.res = isl_val_free(data.res);
+	isl_union_pw_aff_free(upa);
 
 	return data.res;
 }
 
+/* Return the minimum of the integer piecewise affine
+ * expression "upa" over its definition domain.
+ *
+ * Return negative infinity if the optimal value is unbounded and
+ * NaN if the domain of the expression is empty.
+ */
+__isl_give isl_val *isl_union_pw_aff_min_val(__isl_take isl_union_pw_aff *upa)
+{
+	return isl_union_pw_aff_opt_val(upa, 0);
+}
+
+/* Return the maximum of the integer piecewise affine
+ * expression "upa" over its definition domain.
+ *
+ * Return infinity if the optimal value is unbounded and
+ * NaN if the domain of the expression is empty.
+ */
+__isl_give isl_val *isl_union_pw_aff_max_val(__isl_take isl_union_pw_aff *upa)
+{
+	return isl_union_pw_aff_opt_val(upa, 1);
+}
+
+/* Return a list of minima (maxima if "max" is set)
+ * for each of the expressions in "mupa" over their domains.
+ *
+ * An element in the list is infinity or negative infinity if the optimal
+ * value of the corresponding expression is unbounded and
+ * NaN if the domain of the expression is empty.
+ *
+ * Iterate over all the expressions in "mupa" and collect the results.
+ */
+static __isl_give isl_multi_val *isl_multi_union_pw_aff_opt_multi_val(
+	__isl_take isl_multi_union_pw_aff *mupa, int max)
+{
+	int i;
+	isl_size n;
+	isl_multi_val *mv;
+
+	n = isl_multi_union_pw_aff_size(mupa);
+	if (n < 0)
+		mupa = isl_multi_union_pw_aff_free(mupa);
+	if (!mupa)
+		return NULL;
+
+	mv = isl_multi_val_zero(isl_multi_union_pw_aff_get_space(mupa));
+
+	for (i = 0; i < n; ++i) {
+		isl_val *v;
+		isl_union_pw_aff *upa;
+
+		upa = isl_multi_union_pw_aff_get_union_pw_aff(mupa, i);
+		v = isl_union_pw_aff_opt_val(upa, max);
+		mv = isl_multi_val_set_val(mv, i, v);
+	}
+
+	isl_multi_union_pw_aff_free(mupa);
+	return mv;
+}
+
 /* Return a list of minima (maxima if "max" is set) over the points in "uset"
  * for each of the expressions in "obj".
  *
@@ -793,33 +810,15 @@ static __isl_give isl_val *isl_union_set_opt_union_pw_aff(
  * value of the corresponding expression is unbounded and
  * NaN if the intersection of "uset" with the domain of the expression
  * is empty.
- *
- * Iterate over all the expressions in "obj" and collect the results.
  */
 static __isl_give isl_multi_val *isl_union_set_opt_multi_union_pw_aff(
 	__isl_keep isl_union_set *uset, int max,
 	__isl_keep isl_multi_union_pw_aff *obj)
 {
-	int i, n;
-	isl_multi_val *mv;
-
-	if (!uset || !obj)
-		return NULL;
-
-	n = isl_multi_union_pw_aff_dim(obj, isl_dim_set);
-	mv = isl_multi_val_zero(isl_multi_union_pw_aff_get_space(obj));
-
-	for (i = 0; i < n; ++i) {
-		isl_val *v;
-		isl_union_pw_aff *upa;
-
-		upa = isl_multi_union_pw_aff_get_union_pw_aff(obj, i);
-		v = isl_union_set_opt_union_pw_aff(uset, max, upa);
-		isl_union_pw_aff_free(upa);
-		mv = isl_multi_val_set_val(mv, i, v);
-	}
-
-	return mv;
+	uset = isl_union_set_copy(uset);
+	obj = isl_multi_union_pw_aff_copy(obj);
+	obj = isl_multi_union_pw_aff_intersect_domain(obj, uset);
+	return isl_multi_union_pw_aff_opt_multi_val(obj, max);
 }
 
 /* Return a list of minima over the points in "uset"
@@ -835,3 +834,56 @@ __isl_give isl_multi_val *isl_union_set_min_multi_union_pw_aff(
 {
 	return isl_union_set_opt_multi_union_pw_aff(uset, 0, obj);
 }
+
+/* Return a list of minima
+ * for each of the expressions in "mupa" over their domains.
+ *
+ * An element in the list is negative infinity if the optimal
+ * value of the corresponding expression is unbounded and
+ * NaN if the domain of the expression is empty.
+ */
+__isl_give isl_multi_val *isl_multi_union_pw_aff_min_multi_val(
+	__isl_take isl_multi_union_pw_aff *mupa)
+{
+	return isl_multi_union_pw_aff_opt_multi_val(mupa, 0);
+}
+
+/* Return a list of maxima
+ * for each of the expressions in "mupa" over their domains.
+ *
+ * An element in the list is infinity if the optimal
+ * value of the corresponding expression is unbounded and
+ * NaN if the domain of the expression is empty.
+ */
+__isl_give isl_multi_val *isl_multi_union_pw_aff_max_multi_val(
+	__isl_take isl_multi_union_pw_aff *mupa)
+{
+	return isl_multi_union_pw_aff_opt_multi_val(mupa, 1);
+}
+
+/* Return the maximal value attained by the given set dimension,
+ * independently of the parameter values and of any other dimensions.
+ *
+ * Return infinity if the optimal value is unbounded and
+ * NaN if "bset" is empty.
+ */
+__isl_give isl_val *isl_basic_set_dim_max_val(__isl_take isl_basic_set *bset,
+	int pos)
+{
+	isl_local_space *ls;
+	isl_aff *obj;
+	isl_val *v;
+
+	if (isl_basic_set_check_range(bset, isl_dim_set, pos, 1) < 0)
+		goto error;
+	ls = isl_local_space_from_space(isl_basic_set_get_space(bset));
+	obj = isl_aff_var_on_domain(ls, isl_dim_set, pos);
+	v = isl_basic_set_max_val(bset, obj);
+	isl_aff_free(obj);
+	isl_basic_set_free(bset);
+
+	return v;
+error:
+	isl_basic_set_free(bset);
+	return NULL;
+}
