@@ -11,12 +11,14 @@
  */
 
 #include <isl_ctx_private.h>
+#include <isl/id.h>
 #include <isl_map_private.h>
 #include <isl_local_space_private.h>
 #include <isl_space_private.h>
 #include <isl_mat_private.h>
 #include <isl_aff_private.h>
 #include <isl_vec_private.h>
+#include <isl_point_private.h>
 #include <isl_seq.h>
 #include <isl_local.h>
 
@@ -43,47 +45,49 @@ uint32_t isl_local_space_get_hash(__isl_keep isl_local_space *ls)
 	return hash;
 }
 
-__isl_give isl_local_space *isl_local_space_alloc_div(__isl_take isl_space *dim,
-	__isl_take isl_mat *div)
+__isl_give isl_local_space *isl_local_space_alloc_div(
+	__isl_take isl_space *space, __isl_take isl_mat *div)
 {
 	isl_ctx *ctx;
 	isl_local_space *ls = NULL;
 
-	if (!dim || !div)
+	if (!space || !div)
 		goto error;
 
-	ctx = isl_space_get_ctx(dim);
+	ctx = isl_space_get_ctx(space);
 	ls = isl_calloc_type(ctx, struct isl_local_space);
 	if (!ls)
 		goto error;
 
 	ls->ref = 1;
-	ls->dim = dim;
+	ls->dim = space;
 	ls->div = div;
 
 	return ls;
 error:
 	isl_mat_free(div);
-	isl_space_free(dim);
+	isl_space_free(space);
 	isl_local_space_free(ls);
 	return NULL;
 }
 
-__isl_give isl_local_space *isl_local_space_alloc(__isl_take isl_space *dim,
+__isl_give isl_local_space *isl_local_space_alloc(__isl_take isl_space *space,
 	unsigned n_div)
 {
 	isl_ctx *ctx;
 	isl_mat *div;
-	unsigned total;
+	isl_size total;
 
-	if (!dim)
+	if (!space)
 		return NULL;
 
-	total = isl_space_dim(dim, isl_dim_all);
+	total = isl_space_dim(space, isl_dim_all);
+	if (total < 0)
+		return isl_local_space_from_space(isl_space_free(space));
 
-	ctx = isl_space_get_ctx(dim);
+	ctx = isl_space_get_ctx(space);
 	div = isl_mat_alloc(ctx, n_div, 1 + 1 + total + n_div);
-	return isl_local_space_alloc_div(dim, div);
+	return isl_local_space_alloc_div(space, div);
 }
 
 __isl_give isl_local_space *isl_local_space_from_space(__isl_take isl_space *dim)
@@ -165,6 +169,30 @@ isl_bool isl_local_space_has_equal_space(__isl_keep isl_local_space *ls1,
 	return isl_space_is_equal(ls1->dim, ls2->dim);
 }
 
+/* Is the space of "ls" equal to "space"?
+ */
+isl_bool isl_local_space_has_space(__isl_keep isl_local_space *ls,
+	__isl_keep isl_space *space)
+{
+	return isl_space_is_equal(isl_local_space_peek_space(ls), space);
+}
+
+/* Check that the space of "ls" is equal to "space".
+ */
+static isl_stat isl_local_space_check_has_space(__isl_keep isl_local_space *ls,
+	__isl_keep isl_space *space)
+{
+	isl_bool ok;
+
+	ok = isl_local_space_has_space(ls, space);
+	if (ok < 0)
+		return isl_stat_error;
+	if (!ok)
+		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
+			"spaces don't match", return isl_stat_error);
+	return isl_stat_ok;
+}
+
 /* Return true if the two local spaces are identical, with identical
  * expressions for the integer divisions.
  */
@@ -209,33 +237,42 @@ int isl_local_space_cmp(__isl_keep isl_local_space *ls1,
 	return isl_local_cmp(ls1->div, ls2->div);
 }
 
-int isl_local_space_dim(__isl_keep isl_local_space *ls,
+isl_size isl_local_space_dim(__isl_keep isl_local_space *ls,
 	enum isl_dim_type type)
 {
 	if (!ls)
-		return 0;
+		return isl_size_error;
 	if (type == isl_dim_div)
 		return ls->div->n_row;
-	if (type == isl_dim_all)
-		return isl_space_dim(ls->dim, isl_dim_all) + ls->div->n_row;
+	if (type == isl_dim_all) {
+		isl_size dim = isl_space_dim(ls->dim, isl_dim_all);
+		if (dim < 0)
+			return isl_size_error;
+		return dim + ls->div->n_row;
+	}
 	return isl_space_dim(ls->dim, type);
 }
 
+#undef TYPE
+#define TYPE	isl_local_space
+#include "check_type_range_templ.c"
+
 unsigned isl_local_space_offset(__isl_keep isl_local_space *ls,
 	enum isl_dim_type type)
 {
-	isl_space *dim;
+	isl_space *space;
 
 	if (!ls)
 		return 0;
 
-	dim = ls->dim;
+	space = ls->dim;
 	switch (type) {
 	case isl_dim_cst:	return 0;
 	case isl_dim_param:	return 1;
-	case isl_dim_in:	return 1 + dim->nparam;
-	case isl_dim_out:	return 1 + dim->nparam + dim->n_in;
-	case isl_dim_div:	return 1 + dim->nparam + dim->n_in + dim->n_out;
+	case isl_dim_in:	return 1 + space->nparam;
+	case isl_dim_out:	return 1 + space->nparam + space->n_in;
+	case isl_dim_div:
+		return 1 + space->nparam + space->n_in + space->n_out;
 	default:		return 0;
 	}
 }
@@ -304,12 +341,15 @@ static __isl_give isl_aff *extract_div(__isl_keep isl_local_space *ls, int pos)
 static __isl_give isl_aff *drop_unknown_divs_and_extract_div(
 	__isl_keep isl_local_space *ls, int pos)
 {
-	int i, n;
+	int i;
+	isl_size n;
 	isl_bool unknown;
 	isl_aff *aff;
 
-	ls = isl_local_space_copy(ls);
 	n = isl_local_space_dim(ls, isl_dim_div);
+	if (n < 0)
+		return NULL;
+	ls = isl_local_space_copy(ls);
 	for (i = n - 1; i >= 0; --i) {
 		unknown = isl_local_space_div_is_marked_unknown(ls, i);
 		if (unknown < 0)
@@ -363,12 +403,19 @@ __isl_give isl_aff *isl_local_space_get_div(__isl_keep isl_local_space *ls,
 		return drop_unknown_divs_and_extract_div(ls, pos);
 }
 
-__isl_give isl_space *isl_local_space_get_space(__isl_keep isl_local_space *ls)
+/* Return the space of "ls".
+ */
+__isl_keep isl_space *isl_local_space_peek_space(__isl_keep isl_local_space *ls)
 {
 	if (!ls)
 		return NULL;
 
-	return isl_space_copy(ls->dim);
+	return ls->dim;
+}
+
+__isl_give isl_space *isl_local_space_get_space(__isl_keep isl_local_space *ls)
+{
+	return isl_space_copy(isl_local_space_peek_space(ls));
 }
 
 /* Return the space of "ls".
@@ -422,6 +469,13 @@ error:
 	return NULL;
 }
 
+/* Return the local variables of "ls".
+ */
+__isl_keep isl_local *isl_local_space_peek_local(__isl_keep isl_local_space *ls)
+{
+	return ls ? ls->div : NULL;
+}
+
 /* Replace the identifier of the tuple of type "type" by "id".
  */
 __isl_give isl_local_space *isl_local_space_set_tuple_id(
@@ -502,42 +556,6 @@ error:
 	return NULL;
 }
 
-/* Reorder the columns of the given div definitions according to the
- * given reordering.
- * The order of the divs themselves is assumed not to change.
- */
-static __isl_give isl_mat *reorder_divs(__isl_take isl_mat *div,
-	__isl_take isl_reordering *r)
-{
-	int i, j;
-	isl_mat *mat;
-	int extra;
-
-	if (!div || !r)
-		goto error;
-
-	extra = isl_space_dim(r->dim, isl_dim_all) + div->n_row - r->len;
-	mat = isl_mat_alloc(div->ctx, div->n_row, div->n_col + extra);
-	if (!mat)
-		goto error;
-
-	for (i = 0; i < div->n_row; ++i) {
-		isl_seq_cpy(mat->row[i], div->row[i], 2);
-		isl_seq_clr(mat->row[i] + 2, mat->n_col - 2);
-		for (j = 0; j < r->len; ++j)
-			isl_int_set(mat->row[i][2 + r->pos[j]],
-				    div->row[i][2 + j]);
-	}
-
-	isl_reordering_free(r);
-	isl_mat_free(div);
-	return mat;
-error:
-	isl_reordering_free(r);
-	isl_mat_free(div);
-	return NULL;
-}
-
 /* Reorder the dimensions of "ls" according to the given reordering.
  * The reordering r is assumed to have been extended with the local
  * variables, leaving them in the same order.
@@ -549,11 +567,11 @@ __isl_give isl_local_space *isl_local_space_realign(
 	if (!ls || !r)
 		goto error;
 
-	ls->div = reorder_divs(ls->div, isl_reordering_copy(r));
+	ls->div = isl_local_reorder(ls->div, isl_reordering_copy(r));
 	if (!ls->div)
 		goto error;
 
-	ls = isl_local_space_reset_space(ls, isl_space_copy(r->dim));
+	ls = isl_local_space_reset_space(ls, isl_reordering_get_space(r));
 
 	isl_reordering_free(r);
 	return ls;
@@ -681,7 +699,7 @@ __isl_give isl_basic_map *isl_basic_map_sort_divs(
 	__isl_take isl_basic_map *bmap)
 {
 	int i, j;
-	unsigned total;
+	isl_size total;
 
 	bmap = isl_basic_map_order_divs(bmap);
 	if (!bmap)
@@ -689,12 +707,16 @@ __isl_give isl_basic_map *isl_basic_map_sort_divs(
 	if (bmap->n_div <= 1)
 		return bmap;
 
-	total = 2 + isl_basic_map_total_dim(bmap);
+	total = isl_basic_map_dim(bmap, isl_dim_all);
+	if (total < 0)
+		return isl_basic_map_free(bmap);
 	for (i = 1; i < bmap->n_div; ++i) {
 		for (j = i - 1; j >= 0; --j) {
-			if (bmap_cmp_row(bmap, j, j + 1, total) <= 0)
+			if (bmap_cmp_row(bmap, j, j + 1, 2 + total) <= 0)
 				break;
-			isl_basic_map_swap_div(bmap, j, j + 1);
+			bmap = isl_basic_map_swap_div(bmap, j, j + 1);
+			if (!bmap)
+				return NULL;
 		}
 	}
 
@@ -876,25 +898,20 @@ isl_bool isl_local_space_div_is_known(__isl_keep isl_local_space *ls, int div)
  */
 isl_bool isl_local_space_divs_known(__isl_keep isl_local_space *ls)
 {
-	int i;
-
 	if (!ls)
 		return isl_bool_error;
-
-	for (i = 0; i < ls->div->n_row; ++i) {
-		isl_bool unknown = isl_local_space_div_is_marked_unknown(ls, i);
-		if (unknown < 0 || unknown)
-			return isl_bool_not(unknown);
-	}
-
-	return isl_bool_true;
+	return isl_local_divs_known(ls->div);
 }
 
 __isl_give isl_local_space *isl_local_space_domain(
 	__isl_take isl_local_space *ls)
 {
-	ls = isl_local_space_drop_dims(ls, isl_dim_out,
-					0, isl_local_space_dim(ls, isl_dim_out));
+	isl_size n_out;
+
+	n_out = isl_local_space_dim(ls, isl_dim_out);
+	if (n_out < 0)
+		return isl_local_space_free(ls);
+	ls = isl_local_space_drop_dims(ls, isl_dim_out, 0, n_out);
 	ls = isl_local_space_cow(ls);
 	if (!ls)
 		return NULL;
@@ -907,8 +924,12 @@ __isl_give isl_local_space *isl_local_space_domain(
 __isl_give isl_local_space *isl_local_space_range(
 	__isl_take isl_local_space *ls)
 {
-	ls = isl_local_space_drop_dims(ls, isl_dim_in,
-					0, isl_local_space_dim(ls, isl_dim_in));
+	isl_size n_in;
+
+	n_in = isl_local_space_dim(ls, isl_dim_in);
+	if (n_in < 0)
+		return isl_local_space_free(ls);
+	ls = isl_local_space_drop_dims(ls, isl_dim_in, 0, n_in);
 	ls = isl_local_space_cow(ls);
 	if (!ls)
 		return NULL;
@@ -937,17 +958,86 @@ __isl_give isl_local_space *isl_local_space_from_domain(
 __isl_give isl_local_space *isl_local_space_add_dims(
 	__isl_take isl_local_space *ls, enum isl_dim_type type, unsigned n)
 {
-	int pos;
+	isl_size pos;
 
-	if (!ls)
-		return NULL;
 	pos = isl_local_space_dim(ls, type);
+	if (pos < 0)
+		return isl_local_space_free(ls);
 	return isl_local_space_insert_dims(ls, type, pos, n);
 }
 
+/* Lift the basic set "bset", living in the space of "ls"
+ * to live in a space with extra coordinates corresponding
+ * to the local variables of "ls".
+ */
+__isl_give isl_basic_set *isl_local_space_lift_basic_set(
+	__isl_take isl_local_space *ls, __isl_take isl_basic_set *bset)
+{
+	isl_size n_local;
+	isl_space *space;
+	isl_basic_set *ls_bset;
+
+	n_local = isl_local_space_dim(ls, isl_dim_div);
+	space = isl_basic_set_peek_space(bset);
+	if (n_local < 0 ||
+	    isl_local_space_check_has_space(ls, space) < 0)
+		goto error;
+
+	if (n_local == 0) {
+		isl_local_space_free(ls);
+		return bset;
+	}
+
+	bset = isl_basic_set_add_dims(bset, isl_dim_set, n_local);
+	ls_bset = isl_basic_set_from_local_space(ls);
+	ls_bset = isl_basic_set_lift(ls_bset);
+	ls_bset = isl_basic_set_flatten(ls_bset);
+	bset = isl_basic_set_intersect(bset, ls_bset);
+
+	return bset;
+error:
+	isl_local_space_free(ls);
+	isl_basic_set_free(bset);
+	return NULL;
+}
+
+/* Lift the set "set", living in the space of "ls"
+ * to live in a space with extra coordinates corresponding
+ * to the local variables of "ls".
+ */
+__isl_give isl_set *isl_local_space_lift_set(__isl_take isl_local_space *ls,
+	__isl_take isl_set *set)
+{
+	isl_size n_local;
+	isl_basic_set *bset;
+
+	n_local = isl_local_space_dim(ls, isl_dim_div);
+	if (n_local < 0 ||
+	    isl_local_space_check_has_space(ls, isl_set_peek_space(set)) < 0)
+		goto error;
+
+	if (n_local == 0) {
+		isl_local_space_free(ls);
+		return set;
+	}
+
+	set = isl_set_add_dims(set, isl_dim_set, n_local);
+	bset = isl_basic_set_from_local_space(ls);
+	bset = isl_basic_set_lift(bset);
+	bset = isl_basic_set_flatten(bset);
+	set = isl_set_intersect(set, isl_set_from_basic_set(bset));
+
+	return set;
+error:
+	isl_local_space_free(ls);
+	isl_set_free(set);
+	return NULL;
+}
+
 /* Remove common factor of non-constant terms and denominator.
  */
-static void normalize_div(__isl_keep isl_local_space *ls, int div)
+static __isl_give isl_local_space *normalize_div(
+	__isl_take isl_local_space *ls, int div)
 {
 	isl_ctx *ctx = ls->div->ctx;
 	unsigned total = ls->div->n_col - 2;
@@ -956,7 +1046,7 @@ static void normalize_div(__isl_keep isl_local_space *ls, int div)
 	isl_int_gcd(ctx->normalize_gcd,
 		    ctx->normalize_gcd, ls->div->row[div][0]);
 	if (isl_int_is_one(ctx->normalize_gcd))
-		return;
+		return ls;
 
 	isl_seq_scale_down(ls->div->row[div] + 2, ls->div->row[div] + 2,
 			    ctx->normalize_gcd, total);
@@ -964,6 +1054,8 @@ static void normalize_div(__isl_keep isl_local_space *ls, int div)
 			    ctx->normalize_gcd);
 	isl_int_fdiv_q(ls->div->row[div][1], ls->div->row[div][1],
 			    ctx->normalize_gcd);
+
+	return ls;
 }
 
 /* Exploit the equalities in "eq" to simplify the expressions of
@@ -975,14 +1067,17 @@ __isl_give isl_local_space *isl_local_space_substitute_equalities(
 	__isl_take isl_local_space *ls, __isl_take isl_basic_set *eq)
 {
 	int i, j, k;
-	unsigned total;
+	isl_size total, dim;
 	unsigned n_div;
 
 	if (!ls || !eq)
 		goto error;
 
 	total = isl_space_dim(eq->dim, isl_dim_all);
-	if (isl_local_space_dim(ls, isl_dim_all) != total)
+	dim = isl_local_space_dim(ls, isl_dim_all);
+	if (dim < 0 || total < 0)
+		goto error;
+	if (dim != total)
 		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
 			"spaces don't match", goto error);
 	total++;
@@ -1003,7 +1098,9 @@ __isl_give isl_local_space *isl_local_space_substitute_equalities(
 				goto error;
 			isl_seq_elim(ls->div->row[k] + 1, eq->eq[i], j, total,
 					&ls->div->row[k][0]);
-			normalize_div(ls, k);
+			ls = normalize_div(ls, k);
+			if (!ls)
+				goto error;
 		}
 	}
 
@@ -1060,7 +1157,9 @@ __isl_give isl_local_space *isl_local_space_substitute_seq(
 			continue;
 		isl_seq_substitute(ls->div->row[i], pos, subs,
 			ls->div->n_col, subs_len, v);
-		normalize_div(ls, i);
+		ls = normalize_div(ls, i);
+		if (!ls)
+			break;
 	}
 	isl_int_clear(v);
 
@@ -1086,6 +1185,8 @@ __isl_give isl_local_space *isl_local_space_substitute(
 	__isl_take isl_local_space *ls,
 	enum isl_dim_type type, unsigned pos, __isl_keep isl_aff *subs)
 {
+	isl_size n_div;
+
 	ls = isl_local_space_cow(ls);
 	if (!ls || !subs)
 		return isl_local_space_free(ls);
@@ -1093,7 +1194,10 @@ __isl_give isl_local_space *isl_local_space_substitute(
 	if (!isl_space_is_equal(ls->dim, subs->ls->dim))
 		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
 			"spaces don't match", return isl_local_space_free(ls));
-	if (isl_local_space_dim(subs->ls, isl_dim_div) != 0)
+	n_div = isl_local_space_dim(subs->ls, isl_dim_div);
+	if (n_div < 0)
+		return isl_local_space_free(ls);
+	if (n_div != 0)
 		isl_die(isl_local_space_get_ctx(ls), isl_error_unsupported,
 			"cannot handle divs yet",
 			return isl_local_space_free(ls));
@@ -1114,17 +1218,13 @@ __isl_give isl_local_space *isl_local_space_drop_dims(
 	__isl_take isl_local_space *ls,
 	enum isl_dim_type type, unsigned first, unsigned n)
 {
-	isl_ctx *ctx;
-
 	if (!ls)
 		return NULL;
 	if (n == 0 && !isl_local_space_is_named_or_nested(ls, type))
 		return ls;
 
-	ctx = isl_local_space_get_ctx(ls);
-	if (first + n > isl_local_space_dim(ls, type))
-		isl_die(ctx, isl_error_invalid, "range out of bounds",
-			return isl_local_space_free(ls));
+	if (isl_local_space_check_range(ls, type, first, n) < 0)
+		return isl_local_space_free(ls);
 
 	ls = isl_local_space_cow(ls);
 	if (!ls)
@@ -1150,17 +1250,13 @@ __isl_give isl_local_space *isl_local_space_insert_dims(
 	__isl_take isl_local_space *ls,
 	enum isl_dim_type type, unsigned first, unsigned n)
 {
-	isl_ctx *ctx;
-
 	if (!ls)
 		return NULL;
 	if (n == 0 && !isl_local_space_is_named_or_nested(ls, type))
 		return ls;
 
-	ctx = isl_local_space_get_ctx(ls);
-	if (first > isl_local_space_dim(ls, type))
-		isl_die(ctx, isl_error_invalid, "position out of bounds",
-			return isl_local_space_free(ls));
+	if (isl_local_space_check_range(ls, type, first, 0) < 0)
+		return isl_local_space_free(ls);
 
 	ls = isl_local_space_cow(ls);
 	if (!ls)
@@ -1182,6 +1278,49 @@ __isl_give isl_local_space *isl_local_space_insert_dims(
 	return ls;
 }
 
+/* Does the linear part of "constraint" correspond to
+ * integer division "div" in "ls"?
+ *
+ * That is, given div = floor((c + f)/m), is the constraint of the form
+ *
+ *		f - m d + c' >= 0		[sign = 1]
+ * or
+ *		-f + m d + c'' >= 0		[sign = -1]
+ * ?
+ * If so, set *sign to the corresponding value.
+ */
+static isl_bool is_linear_div_constraint(__isl_keep isl_local_space *ls,
+	isl_int *constraint, unsigned div, int *sign)
+{
+	isl_bool unknown;
+	unsigned pos;
+
+	unknown = isl_local_space_div_is_marked_unknown(ls, div);
+	if (unknown < 0)
+		return isl_bool_error;
+	if (unknown)
+		return isl_bool_false;
+
+	pos = isl_local_space_offset(ls, isl_dim_div) + div;
+
+	if (isl_int_eq(constraint[pos], ls->div->row[div][0])) {
+		*sign = -1;
+		if (!isl_seq_is_neg(constraint + 1,
+				    ls->div->row[div] + 2, pos - 1))
+			return isl_bool_false;
+	} else if (isl_int_abs_eq(constraint[pos], ls->div->row[div][0])) {
+		*sign = 1;
+		if (!isl_seq_eq(constraint + 1, ls->div->row[div] + 2, pos - 1))
+			return isl_bool_false;
+	} else {
+		return isl_bool_false;
+	}
+	if (isl_seq_first_non_zero(constraint + pos + 1,
+				    ls->div->n_row - div - 1) != -1)
+		return isl_bool_false;
+	return isl_bool_true;
+}
+
 /* Check if the constraints pointed to by "constraint" is a div
  * constraint corresponding to div "div" in "ls".
  *
@@ -1190,46 +1329,67 @@ __isl_give isl_local_space *isl_local_space_insert_dims(
  *		f - m d >= 0
  * or
  *		-(f-(m-1)) + m d >= 0
+ *
+ * First check if the linear part is of the right form and
+ * then check the constant term.
  */
 isl_bool isl_local_space_is_div_constraint(__isl_keep isl_local_space *ls,
 	isl_int *constraint, unsigned div)
 {
-	unsigned pos;
+	int sign;
+	isl_bool linear;
 
-	if (!ls)
-		return isl_bool_error;
+	linear = is_linear_div_constraint(ls, constraint, div, &sign);
+	if (linear < 0 || !linear)
+		return linear;
 
-	if (isl_int_is_zero(ls->div->row[div][0]))
-		return isl_bool_false;
-
-	pos = isl_local_space_offset(ls, isl_dim_div) + div;
-
-	if (isl_int_eq(constraint[pos], ls->div->row[div][0])) {
+	if (sign < 0) {
 		int neg;
 		isl_int_sub(ls->div->row[div][1],
 				ls->div->row[div][1], ls->div->row[div][0]);
 		isl_int_add_ui(ls->div->row[div][1], ls->div->row[div][1], 1);
-		neg = isl_seq_is_neg(constraint, ls->div->row[div]+1, pos);
+		neg = isl_seq_is_neg(constraint, ls->div->row[div] + 1, 1);
 		isl_int_sub_ui(ls->div->row[div][1], ls->div->row[div][1], 1);
 		isl_int_add(ls->div->row[div][1],
 				ls->div->row[div][1], ls->div->row[div][0]);
 		if (!neg)
 			return isl_bool_false;
-		if (isl_seq_first_non_zero(constraint+pos+1,
-					    ls->div->n_row-div-1) != -1)
+	} else {
+		if (!isl_int_eq(constraint[0], ls->div->row[div][1]))
 			return isl_bool_false;
-	} else if (isl_int_abs_eq(constraint[pos], ls->div->row[div][0])) {
-		if (!isl_seq_eq(constraint, ls->div->row[div]+1, pos))
-			return isl_bool_false;
-		if (isl_seq_first_non_zero(constraint+pos+1,
-					    ls->div->n_row-div-1) != -1)
-			return isl_bool_false;
-	} else
-		return isl_bool_false;
+	}
 
 	return isl_bool_true;
 }
 
+/* Is the constraint pointed to by "constraint" one
+ * of an equality that corresponds to integer division "div" in "ls"?
+ *
+ * That is, given an integer division of the form
+ *
+ *	a = floor((f + c)/m)
+ *
+ * is the equality of the form
+ *
+ *		-f + m d + c' = 0
+ * ?
+ * Note that the constant term is not checked explicitly, but given
+ * that this is a valid equality constraint, the constant c' necessarily
+ * has a value close to -c.
+ */
+isl_bool isl_local_space_is_div_equality(__isl_keep isl_local_space *ls,
+	isl_int *constraint, unsigned div)
+{
+	int sign;
+	isl_bool linear;
+
+	linear = is_linear_div_constraint(ls, constraint, div, &sign);
+	if (linear < 0 || !linear)
+		return linear;
+
+	return isl_bool_ok(sign < 0);
+}
+
 /*
  * Set active[i] to 1 if the dimension at position i is involved
  * in the linear expression l.
@@ -1239,11 +1399,13 @@ int *isl_local_space_get_active(__isl_keep isl_local_space *ls, isl_int *l)
 	int i, j;
 	isl_ctx *ctx;
 	int *active = NULL;
-	unsigned total;
+	isl_size total;
 	unsigned offset;
 
 	ctx = isl_local_space_get_ctx(ls);
 	total = isl_local_space_dim(ls, isl_dim_all);
+	if (total < 0)
+		return NULL;
 	active = isl_calloc_array(ctx, int, total);
 	if (total && !active)
 		return NULL;
@@ -1333,7 +1495,7 @@ __isl_give isl_local_space *isl_local_space_preimage_multi_aff(
 	int i;
 	isl_space *space;
 	isl_local_space *res = NULL;
-	int n_div_ls, n_div_ma;
+	isl_size n_div_ls, n_div_ma;
 	isl_int f, c1, c2, g;
 
 	ma = isl_multi_aff_align_divs(ma);
@@ -1344,7 +1506,9 @@ __isl_give isl_local_space *isl_local_space_preimage_multi_aff(
 			"spaces don't match", goto error);
 
 	n_div_ls = isl_local_space_dim(ls, isl_dim_div);
-	n_div_ma = ma->n ? isl_aff_dim(ma->p[0], isl_dim_div) : 0;
+	n_div_ma = ma->n ? isl_aff_dim(ma->u.p[0], isl_dim_div) : 0;
+	if (n_div_ls < 0 || n_div_ma < 0)
+		goto error;
 
 	space = isl_space_domain(isl_multi_aff_get_space(ma));
 	res = isl_local_space_alloc(space, n_div_ma + n_div_ls);
@@ -1353,7 +1517,7 @@ __isl_give isl_local_space *isl_local_space_preimage_multi_aff(
 
 	if (n_div_ma) {
 		isl_mat_free(res->div);
-		res->div = isl_mat_copy(ma->p[0]->ls->div);
+		res->div = isl_mat_copy(ma->u.p[0]->ls->div);
 		res->div = isl_mat_add_zero_cols(res->div, n_div_ls);
 		res->div = isl_mat_add_rows(res->div, n_div_ls);
 		if (!res->div)
@@ -1370,9 +1534,13 @@ __isl_give isl_local_space *isl_local_space_preimage_multi_aff(
 			isl_int_set_si(res->div->row[n_div_ma + i][0], 0);
 			continue;
 		}
-		isl_seq_preimage(res->div->row[n_div_ma + i], ls->div->row[i],
-				ma, 0, 0, n_div_ma, n_div_ls, f, c1, c2, g, 1);
-		normalize_div(res, n_div_ma + i);
+		if (isl_seq_preimage(res->div->row[n_div_ma + i],
+			    ls->div->row[i],
+			    ma, 0, 0, n_div_ma, n_div_ls, f, c1, c2, g, 1) < 0)
+			res = isl_local_space_free(res);
+		res = normalize_div(res, n_div_ma + i);
+		if (!res)
+			break;
 	}
 
 	isl_int_clear(f);
@@ -1411,13 +1579,10 @@ __isl_give isl_local_space *isl_local_space_move_dims(
 	    !isl_local_space_is_named_or_nested(ls, dst_type))
 		return ls;
 
-	if (src_pos + n > isl_local_space_dim(ls, src_type))
-		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
-			"range out of bounds", return isl_local_space_free(ls));
-	if (dst_pos > isl_local_space_dim(ls, dst_type))
-		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
-			"position out of bounds",
-			return isl_local_space_free(ls));
+	if (isl_local_space_check_range(ls, src_type, src_pos, n) < 0)
+		return isl_local_space_free(ls);
+	if (isl_local_space_check_range(ls, dst_type, dst_pos, 0) < 0)
+		return isl_local_space_free(ls);
 	if (src_type == isl_dim_div)
 		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
 			"cannot move divs", return isl_local_space_free(ls));
@@ -1514,3 +1679,41 @@ __isl_give isl_local_space *isl_local_space_wrap(__isl_take isl_local_space *ls)
 
 	return ls;
 }
+
+/* Lift the point "pnt", living in the space of "ls"
+ * to live in a space with extra coordinates corresponding
+ * to the local variables of "ls".
+ */
+__isl_give isl_point *isl_local_space_lift_point(__isl_take isl_local_space *ls,
+	__isl_take isl_point *pnt)
+{
+	isl_size n_local;
+	isl_space *space;
+	isl_local *local;
+	isl_vec *vec;
+
+	if (isl_local_space_check_has_space(ls, isl_point_peek_space(pnt)) < 0)
+		goto error;
+
+	local = isl_local_space_peek_local(ls);
+	n_local = isl_local_space_dim(ls, isl_dim_div);
+	if (n_local < 0)
+		goto error;
+
+	space = isl_point_take_space(pnt);
+	vec = isl_point_take_vec(pnt);
+
+	space = isl_space_lift(space, n_local);
+	vec = isl_local_extend_point_vec(local, vec);
+
+	pnt = isl_point_restore_vec(pnt, vec);
+	pnt = isl_point_restore_space(pnt, space);
+
+	isl_local_space_free(ls);
+
+	return pnt;
+error:
+	isl_local_space_free(ls);
+	isl_point_free(pnt);
+	return NULL;
+}
