@@ -11,8 +11,8 @@
 #include <isl_ctx_private.h>
 #include <isl_val_private.h>
 
-#undef BASE
-#define BASE val
+#undef EL_BASE
+#define EL_BASE val
 
 #include <isl_list_templ.c>
 
@@ -295,15 +295,15 @@ long isl_val_get_num_si(__isl_keep isl_val *v)
  *
  * If "v" is not a rational value, then the result is undefined.
  */
-int isl_val_get_num_isl_int(__isl_keep isl_val *v, isl_int *n)
+isl_stat isl_val_get_num_isl_int(__isl_keep isl_val *v, isl_int *n)
 {
 	if (!v)
-		return -1;
+		return isl_stat_error;
 	if (!isl_val_is_rat(v))
 		isl_die(isl_val_get_ctx(v), isl_error_invalid,
-			"expecting rational value", return -1);
+			"expecting rational value", return isl_stat_error);
 	isl_int_set(*n, v->n);
-	return 0;
+	return isl_stat_ok;
 }
 
 /* Extract the denominator of a rational value "v" as an integer.
@@ -528,7 +528,7 @@ __isl_give isl_val *isl_val_trunc(__isl_take isl_val *v)
 
 /* Return 2^v, where v is an integer (that is not too large).
  */
-__isl_give isl_val *isl_val_2exp(__isl_take isl_val *v)
+__isl_give isl_val *isl_val_pow2(__isl_take isl_val *v)
 {
 	unsigned long exp;
 	int neg;
@@ -557,6 +557,13 @@ __isl_give isl_val *isl_val_2exp(__isl_take isl_val *v)
 	return v;
 }
 
+/* This is an alternative name for the function above.
+ */
+__isl_give isl_val *isl_val_2exp(__isl_take isl_val *v)
+{
+	return isl_val_pow2(v);
+}
+
 /* Return the minimum of "v1" and "v2".
  */
 __isl_give isl_val *isl_val_min(__isl_take isl_val *v1, __isl_take isl_val *v2)
@@ -962,7 +969,7 @@ isl_bool isl_val_is_divisible_by(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 		isl_die(isl_val_get_ctx(v1), isl_error_invalid,
 			"expecting two integers", return isl_bool_error);
 
-	return isl_int_is_divisible_by(v1->n, v2->n);
+	return isl_bool_ok(isl_int_is_divisible_by(v1->n, v2->n));
 }
 
 /* Given two integer values "v1" and "v2", return the residue of "v1"
@@ -1135,7 +1142,7 @@ isl_bool isl_val_is_int(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_one(v->d);
+	return isl_bool_ok(isl_int_is_one(v->d));
 }
 
 /* Does "v" represent a rational value?
@@ -1145,7 +1152,7 @@ isl_bool isl_val_is_rat(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return !isl_int_is_zero(v->d);
+	return isl_bool_ok(!isl_int_is_zero(v->d));
 }
 
 /* Does "v" represent NaN?
@@ -1155,7 +1162,7 @@ isl_bool isl_val_is_nan(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_zero(v->n) && isl_int_is_zero(v->d);
+	return isl_bool_ok(isl_int_is_zero(v->n) && isl_int_is_zero(v->d));
 }
 
 /* Does "v" represent +infinity?
@@ -1165,7 +1172,7 @@ isl_bool isl_val_is_infty(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_pos(v->n) && isl_int_is_zero(v->d);
+	return isl_bool_ok(isl_int_is_pos(v->n) && isl_int_is_zero(v->d));
 }
 
 /* Does "v" represent -infinity?
@@ -1175,7 +1182,7 @@ isl_bool isl_val_is_neginfty(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_neg(v->n) && isl_int_is_zero(v->d);
+	return isl_bool_ok(isl_int_is_neg(v->n) && isl_int_is_zero(v->d));
 }
 
 /* Does "v" represent the integer zero?
@@ -1185,7 +1192,7 @@ isl_bool isl_val_is_zero(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_zero(v->n) && !isl_int_is_zero(v->d);
+	return isl_bool_ok(isl_int_is_zero(v->n) && !isl_int_is_zero(v->d));
 }
 
 /* Does "v" represent the integer one?
@@ -1198,7 +1205,7 @@ isl_bool isl_val_is_one(__isl_keep isl_val *v)
 	if (isl_val_is_nan(v))
 		return isl_bool_false;
 
-	return isl_int_eq(v->n, v->d);
+	return isl_bool_ok(isl_int_eq(v->n, v->d));
 }
 
 /* Does "v" represent the integer negative one?
@@ -1208,7 +1215,7 @@ isl_bool isl_val_is_negone(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_neg(v->n) && isl_int_abs_eq(v->n, v->d);
+	return isl_bool_ok(isl_int_is_neg(v->n) && isl_int_abs_eq(v->n, v->d));
 }
 
 /* Is "v" (strictly) positive?
@@ -1218,7 +1225,7 @@ isl_bool isl_val_is_pos(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_pos(v->n);
+	return isl_bool_ok(isl_int_is_pos(v->n));
 }
 
 /* Is "v" (strictly) negative?
@@ -1228,7 +1235,7 @@ isl_bool isl_val_is_neg(__isl_keep isl_val *v)
 	if (!v)
 		return isl_bool_error;
 
-	return isl_int_is_neg(v->n);
+	return isl_bool_ok(isl_int_is_neg(v->n));
 }
 
 /* Is "v" non-negative?
@@ -1241,7 +1248,7 @@ isl_bool isl_val_is_nonneg(__isl_keep isl_val *v)
 	if (isl_val_is_nan(v))
 		return isl_bool_false;
 
-	return isl_int_is_nonneg(v->n);
+	return isl_bool_ok(isl_int_is_nonneg(v->n));
 }
 
 /* Is "v" non-positive?
@@ -1254,7 +1261,7 @@ isl_bool isl_val_is_nonpos(__isl_keep isl_val *v)
 	if (isl_val_is_nan(v))
 		return isl_bool_false;
 
-	return isl_int_is_nonpos(v->n);
+	return isl_bool_ok(isl_int_is_nonpos(v->n));
 }
 
 /* Return the sign of "v".
@@ -1282,7 +1289,7 @@ isl_bool isl_val_lt(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	if (!v1 || !v2)
 		return isl_bool_error;
 	if (isl_val_is_int(v1) && isl_val_is_int(v2))
-		return isl_int_lt(v1->n, v2->n);
+		return isl_bool_ok(isl_int_lt(v1->n, v2->n));
 	if (isl_val_is_nan(v1) || isl_val_is_nan(v2))
 		return isl_bool_false;
 	if (isl_val_eq(v1, v2))
@@ -1299,7 +1306,7 @@ isl_bool isl_val_lt(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	isl_int_init(t);
 	isl_int_mul(t, v1->n, v2->d);
 	isl_int_submul(t, v2->n, v1->d);
-	lt = isl_int_is_neg(t);
+	lt = isl_bool_ok(isl_int_is_neg(t));
 	isl_int_clear(t);
 
 	return lt;
@@ -1312,6 +1319,31 @@ isl_bool isl_val_gt(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	return isl_val_lt(v2, v1);
 }
 
+/* Is "v" (strictly) greater than "i"?
+ */
+isl_bool isl_val_gt_si(__isl_keep isl_val *v, long i)
+{
+	isl_val *vi;
+	isl_bool res;
+
+	if (!v)
+		return isl_bool_error;
+	if (isl_val_is_int(v))
+		return isl_bool_ok(isl_int_cmp_si(v->n, i) > 0);
+	if (isl_val_is_nan(v))
+		return isl_bool_false;
+	if (isl_val_is_infty(v))
+		return isl_bool_true;
+	if (isl_val_is_neginfty(v))
+		return isl_bool_false;
+
+	vi = isl_val_int_from_si(isl_val_get_ctx(v), i);
+	res = isl_bool_ok(isl_val_gt(v, vi));
+	isl_val_free(vi);
+
+	return res;
+}
+
 /* Is "v1" less than or equal to "v2"?
  */
 isl_bool isl_val_le(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
@@ -1322,7 +1354,7 @@ isl_bool isl_val_le(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	if (!v1 || !v2)
 		return isl_bool_error;
 	if (isl_val_is_int(v1) && isl_val_is_int(v2))
-		return isl_int_le(v1->n, v2->n);
+		return isl_bool_ok(isl_int_le(v1->n, v2->n));
 	if (isl_val_is_nan(v1) || isl_val_is_nan(v2))
 		return isl_bool_false;
 	if (isl_val_eq(v1, v2))
@@ -1339,7 +1371,7 @@ isl_bool isl_val_le(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	isl_int_init(t);
 	isl_int_mul(t, v1->n, v2->d);
 	isl_int_submul(t, v2->n, v1->d);
-	le = isl_int_is_nonpos(t);
+	le = isl_bool_ok(isl_int_is_nonpos(t));
 	isl_int_clear(t);
 
 	return le;
@@ -1392,7 +1424,8 @@ isl_bool isl_val_eq(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	if (isl_val_is_nan(v1) || isl_val_is_nan(v2))
 		return isl_bool_false;
 
-	return isl_int_eq(v1->n, v2->n) && isl_int_eq(v1->d, v2->d);
+	return isl_bool_ok(isl_int_eq(v1->n, v2->n) &&
+			   isl_int_eq(v1->d, v2->d));
 }
 
 /* Is "v1" equal to "v2" in absolute value?
@@ -1404,7 +1437,8 @@ isl_bool isl_val_abs_eq(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	if (isl_val_is_nan(v1) || isl_val_is_nan(v2))
 		return isl_bool_false;
 
-	return isl_int_abs_eq(v1->n, v2->n) && isl_int_eq(v1->d, v2->d);
+	return isl_bool_ok(isl_int_abs_eq(v1->n, v2->n) &&
+			   isl_int_eq(v1->d, v2->d));
 }
 
 /* Is "v1" different from "v2"?
@@ -1416,7 +1450,8 @@ isl_bool isl_val_ne(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
 	if (isl_val_is_nan(v1) || isl_val_is_nan(v2))
 		return isl_bool_false;
 
-	return isl_int_ne(v1->n, v2->n) || isl_int_ne(v1->d, v2->d);
+	return isl_bool_ok(isl_int_ne(v1->n, v2->n) ||
+			   isl_int_ne(v1->d, v2->d));
 }
 
 /* Print a textual representation of "v" onto "p".
@@ -1455,7 +1490,8 @@ __isl_give isl_printer *isl_printer_print_val(__isl_take isl_printer *p,
  * This is a private copy of isl_val_eq for use in the generic
  * isl_multi_*_plain_is_equal instantiated for isl_val.
  */
-int isl_val_plain_is_equal(__isl_keep isl_val *val1, __isl_keep isl_val *val2)
+isl_bool isl_val_plain_is_equal(__isl_keep isl_val *val1,
+	__isl_keep isl_val *val2)
 {
 	return isl_val_eq(val1, val2);
 }
@@ -1466,15 +1502,15 @@ int isl_val_plain_is_equal(__isl_keep isl_val *val1, __isl_keep isl_val *val2)
  * This function is only meant to be used in the generic isl_multi_*
  * functions which have to deal with base objects that have an associated
  * space.  Since an isl_val does not have any coefficients, this function
- * always return 0.
+ * always returns isl_bool_false.
  */
-int isl_val_involves_dims(__isl_keep isl_val *v, enum isl_dim_type type,
+isl_bool isl_val_involves_dims(__isl_keep isl_val *v, enum isl_dim_type type,
 	unsigned first, unsigned n)
 {
 	if (!v)
-		return -1;
+		return isl_bool_error;
 
-	return 0;
+	return isl_bool_false;
 }
 
 /* Insert "n" dimensions of type "type" at position "first".
@@ -1490,19 +1526,6 @@ __isl_give isl_val *isl_val_insert_dims(__isl_take isl_val *v,
 	return v;
 }
 
-/* Drop the "n" first dimensions of type "type" at position "first".
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  Since an isl_val does not have an associated space, this function
- * does not do anything.
- */
-__isl_give isl_val *isl_val_drop_dims(__isl_take isl_val *v,
-	enum isl_dim_type type, unsigned first, unsigned n)
-{
-	return v;
-}
-
 /* Change the name of the dimension of type "type" at position "pos" to "s".
  *
  * This function is only meant to be used in the generic isl_multi_*
@@ -1516,73 +1539,6 @@ __isl_give isl_val *isl_val_set_dim_name(__isl_take isl_val *v,
 	return v;
 }
 
-/* Return the space of "v".
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  The conditions surrounding the call to this function make sure
- * that this function will never actually get called.  We return a valid
- * space anyway, just in case.
- */
-__isl_give isl_space *isl_val_get_space(__isl_keep isl_val *v)
-{
-	if (!v)
-		return NULL;
-
-	return isl_space_params_alloc(isl_val_get_ctx(v), 0);
-}
-
-/* Reset the domain space of "v" to "space".
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  Since an isl_val does not have an associated space, this function
- * does not do anything, apart from error handling and cleaning up memory.
- */
-__isl_give isl_val *isl_val_reset_domain_space(__isl_take isl_val *v,
-	__isl_take isl_space *space)
-{
-	if (!space)
-		return isl_val_free(v);
-	isl_space_free(space);
-	return v;
-}
-
-/* Align the parameters of "v" to those of "space".
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  Since an isl_val does not have an associated space, this function
- * does not do anything, apart from error handling and cleaning up memory.
- * Note that the conditions surrounding the call to this function make sure
- * that this function will never actually get called.
- */
-__isl_give isl_val *isl_val_align_params(__isl_take isl_val *v,
-	__isl_take isl_space *space)
-{
-	if (!space)
-		return isl_val_free(v);
-	isl_space_free(space);
-	return v;
-}
-
-/* Reorder the dimensions of the domain of "v" according
- * to the given reordering.
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  Since an isl_val does not have an associated space, this function
- * does not do anything, apart from error handling and cleaning up memory.
- */
-__isl_give isl_val *isl_val_realign_domain(__isl_take isl_val *v,
-	__isl_take isl_reordering *r)
-{
-	if (!r)
-		return isl_val_free(v);
-	isl_reordering_free(r);
-	return v;
-}
-
 /* Return an isl_val that is zero on "ls".
  *
  * This function is only meant to be used in the generic isl_multi_*
@@ -1601,47 +1557,22 @@ __isl_give isl_val *isl_val_zero_on_domain(__isl_take isl_local_space *ls)
 	return isl_val_zero(ctx);
 }
 
-/* Do the parameters of "v" match those of "space"?
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  Since an isl_val does not have an associated space, this function
- * simply returns true, except if "v" or "space" are NULL.
- */
-isl_bool isl_val_matching_params(__isl_keep isl_val *v,
-	__isl_keep isl_space *space)
-{
-	if (!v || !space)
-		return isl_bool_error;
-	return isl_bool_true;
-}
-
-/* Check that the domain space of "v" matches "space".
- *
- * This function is only meant to be used in the generic isl_multi_*
- * functions which have to deal with base objects that have an associated
- * space.  Since an isl_val does not have an associated space, this function
- * simply returns 0, except if "v" or "space" are NULL.
- */
-isl_stat isl_val_check_match_domain_space(__isl_keep isl_val *v,
-	__isl_keep isl_space *space)
-{
-	if (!v || !space)
-		return isl_stat_error;
-	return isl_stat_ok;
-}
-
 #define isl_val_involves_nan isl_val_is_nan
 
 #undef BASE
 #define BASE val
 
-#define NO_DOMAIN
-#define NO_IDENTITY
-#define NO_FROM_BASE
-#define NO_MOVE_DIMS
+#include <isl_multi_no_domain_templ.c>
+#include <isl_multi_no_explicit_domain.c>
 #include <isl_multi_templ.c>
+#include <isl_multi_arith_templ.c>
+#include <isl_multi_dim_id_templ.c>
 #include <isl_multi_dims.c>
+#include <isl_multi_nan_templ.c>
+#include <isl_multi_product_templ.c>
+#include <isl_multi_splice_templ.c>
+#include <isl_multi_tuple_id_templ.c>
+#include <isl_multi_zero_templ.c>
 
 /* Apply "fn" to each of the elements of "mv" with as second argument "v".
  */
@@ -1658,8 +1589,8 @@ static __isl_give isl_multi_val *isl_multi_val_fn_val(
 		goto error;
 
 	for (i = 0; i < mv->n; ++i) {
-		mv->p[i] = fn(mv->p[i], isl_val_copy(v));
-		if (!mv->p[i])
+		mv->u.p[i] = fn(mv->u.p[i], isl_val_copy(v));
+		if (!mv->u.p[i])
 			goto error;
 	}
 
